package com.diagbot.web;


import com.diagbot.client.bean.SearchData;
import com.diagbot.dto.AIDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.AIFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.vo.AIVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description: AI服务控制层
 * @author: gaodm
 * @time: 2018/10/25 10:09
 */
@RestController
@RequestMapping("/ai")
public class AIController {

    @Autowired
    AIFacade aiFacade;

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 症状相关数据
     */
    @ApiOperation(value = "症状推理症状接口", notes = "症状推理症状接口")
    @PostMapping("/pushSymptom")
    public RespDTO<AIDTO> pushSymptom(@RequestBody @Valid AIVO aivo) {
        SearchData searchData = new SearchData();
        BeanUtil.copyProperties(aivo, searchData);
        searchData.setSysCode("1");
        searchData.setFeatureType("1");
        return RespDTO.onSuc(aiFacade.push(searchData));
    }

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 疾病相关数据
     */
    @ApiOperation(value = "症状推理疾病接口", notes = "症状推理疾病接口")
    @PostMapping("/pushDis")
    public RespDTO<AIDTO> pushDis(@RequestBody @Valid AIVO aivo) {
        SearchData searchData = new SearchData();
        BeanUtil.copyProperties(aivo, searchData);
        searchData.setSysCode("1");
        searchData.setFeatureType("7");
        return RespDTO.onSuc(aiFacade.push(searchData));
    }

}

